@@ -545,14 +545,8 @@ class TrustedCoinPlugin(BasePlugin):
         wizard.choice_dialog(title=title, message=message, choices=choices, run_next=wizard.run)
 
     def choose_seed_type(self, wizard):
-        choices = [
-            ('create_2fa_segwit_seed', _('Segwit 2FA')),
-            ('create_2fa_seed', _('Legacy 2FA')),
-        ]
-        wizard.choose_seed_type(choices=choices)
-
-    def create_2fa_seed(self, wizard): self.create_seed(wizard, '2fa')
-    def create_2fa_segwit_seed(self, wizard): self.create_seed(wizard, '2fa_segwit')
+        seed_type = '2fa' if self.config.get('nosegwit') else '2fa_segwit'
+        self.create_seed(wizard, seed_type)
 
     def create_seed(self, wizard, seed_type):
         seed = self.make_seed(seed_type)
@@ -575,20 +569,25 @@ class TrustedCoinPlugin(BasePlugin):
             raise Exception(f'unexpected seed type: {t}')
         words = seed.split()
         n = len(words)
-        # old version use long seed phrases
-        if n >= 20:
-            # note: pre-2.7 2fa seeds were typically 24-25 words, however they
-            # could probabilistically be arbitrarily shorter due to a bug. (see #3611)
-            # the probability of it being < 20 words is about 2^(-(256+12-19*11)) = 2^(-59)
-            if passphrase != '':
-                raise Exception('old 2fa seed cannot have passphrase')
-            xprv1, xpub1 = self.get_xkeys(' '.join(words[0:12]), t, '', "m/")
-            xprv2, xpub2 = self.get_xkeys(' '.join(words[12:]), t, '', "m/")
-        elif not t == '2fa' or n == 12:
+        if t == '2fa':
+            if n >= 20:  # old scheme
+                # note: pre-2.7 2fa seeds were typically 24-25 words, however they
+                # could probabilistically be arbitrarily shorter due to a bug. (see #3611)
+                # the probability of it being < 20 words is about 2^(-(256+12-19*11)) = 2^(-59)
+                if passphrase != '':
+                    raise Exception('old 2fa seed cannot have passphrase')
+                xprv1, xpub1 = self.get_xkeys(' '.join(words[0:12]), t, '', "m/")
+                xprv2, xpub2 = self.get_xkeys(' '.join(words[12:]), t, '', "m/")
+            elif n == 12:  # new scheme
+                xprv1, xpub1 = self.get_xkeys(seed, t, passphrase, "m/0'/")
+                xprv2, xpub2 = self.get_xkeys(seed, t, passphrase, "m/1'/")
+            else:
+                raise Exception(f'unrecognized seed length for "2fa" seed: {n}')
+        elif t == '2fa_segwit':
             xprv1, xpub1 = self.get_xkeys(seed, t, passphrase, "m/0'/")
             xprv2, xpub2 = self.get_xkeys(seed, t, passphrase, "m/1'/")
         else:
-            raise Exception('unrecognized seed length: {} words'.format(n))
+            raise Exception(f'unexpected seed type: {t}')
         return xprv1, xpub1, xprv2, xpub2
 
     def create_keystore(self, wizard, seed, passphrase):
